/*
 * Based on The Android Open Source Project
 * com.example.android.networkconnect
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cz.vsb.gis.ruz76.patrac.android.activities;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.app.ActivityCompat;
import android.text.method.ScrollingMovementMethod;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import cz.vsb.gis.ruz76.patrac.android.helpers.AdapterHelper;
import cz.vsb.gis.ruz76.patrac.android.helpers.DownloadFileFromUrl;
import cz.vsb.gis.ruz76.patrac.android.helpers.GetRequestUpdate;
import cz.vsb.gis.ruz76.patrac.android.R;
import cz.vsb.gis.ruz76.patrac.android.helpers.GetRequest;
import cz.vsb.gis.ruz76.patrac.android.domain.Message;
import cz.vsb.gis.ruz76.patrac.android.domain.RequestMode;
import cz.vsb.gis.ruz76.patrac.android.domain.Waypoint;
import cz.vsb.gis.ruz76.patrac.android.helpers.LogHelper;
import cz.vsb.gis.ruz76.patrac.android.helpers.Network;
import cz.vsb.gis.ruz76.patrac.android.helpers.Notificator;

/**
 * Main Activity.
 */
public class MainActivity extends Activity implements LocationListener, GetRequestUpdate {

    private Menu menu;

    public static String sessionId = null;
    public static String searchid;
    public static String arrive = "Nepřijedu";
    public static String endPoint;
    public static String StatusMessages = null;
    public static ArrayList<Waypoint> waypoints;
    public static RequestMode mode = RequestMode.SLEEPING;
    public static boolean sendingLocation = false;
    public static boolean readingMessage = false;

    private TextView mDataText;
    private TextView mStatusText;
    private ListView messagesListView;
    private TextView mWelcomeText;
    private Intent callOnDuty;

    private double lat = 0;
    private double lon = 0;
    private double latFromListener = 0;
    private double lonFromListener = 0;

    private int positionCount = 0;
    private int loggedPositionCount = 0;
    private int sendPositionCount = 0;
    private int messagesCount = 0;
    private int errorsCount = 0;
    private boolean connected = false;

    private int lastLoggedPositionId = 0;

    Timer timerPosition;
    PositionTask myPositionTask;
    Timer timerMessage;
    MessageTask myMessageTask;
    Timer timerSearch;
    SearchTask mySearchTask;
    Timer timerCallOnDuty;
    CallOnDutyTask myCallOnDutyTask;
    SharedPreferences sharedPrefs;

    protected Context context;
    protected LocationManager locationManager;
    private Notificator notificator = new Notificator();

    // Create a List from String Array elements
    private List<String> messages_list;
    private List<Message> messages_list_full;

    // Create an ArrayAdapter from List
    private ArrayAdapter<String> arrayAdapter;

    //Methods from Location Listener
    @Override
    public void onLocationChanged(Location location) throws SecurityException {
        Location locationGPS = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
        if (locationGPS != null) {
            latFromListener = locationGPS.getLatitude();
            lonFromListener = locationGPS.getLongitude();
        }
    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
    }

    @Override
    public void onProviderEnabled(String provider) {
    }

    @Override
    public void onProviderDisabled(String provider) {
    }

    /**
     * Creates instances from saved state.
     * Sets permissions or requests permissions.
     * Sets gui based on layout activity_main.
     *
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) throws SecurityException {
        super.onCreate(savedInstanceState);
        this.requestWindowFeature(Window.FEATURE_ACTION_BAR);
        setPermissions();
        sharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
        setContentView(R.layout.activity_main);
        extractGuiItems();
        mDataText.setText(R.string.mode_sleeping);
        setSearchTimer();
        context = this.getApplicationContext();
        notificator.setContext(context);
        endPoint = sharedPrefs.getString("endpoint", getString(R.string.pref_default_endpoint));
        locationManager = (LocationManager) context.getSystemService(LOCATION_SERVICE);
        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);
        mStatusText.setMovementMethod(new ScrollingMovementMethod());
        mStatusText.setOnLongClickListener(new View.OnLongClickListener() {
            public boolean onLongClick(View arg0) {
                Toast.makeText(getApplicationContext(), "Long Clicked " , Toast.LENGTH_SHORT).show();
                Intent logView = new Intent(MainActivity.this, LogActivity.class);
                startActivity(logView);
                return false;
            }
        });
    }

    /**
     * Ads items to the menu.
     *
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        this.menu = menu;
        return true;
    }

    /**
     * Invoked on menu click.
     *
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.connect_disconnect_action:
                if (!connected) {
                    connect();
                    item.setTitle(getString(R.string.disconnect));
                } else {
                    disconnect();
                    item.setTitle(getString(R.string.connect));
                }
                return true;
            case R.id.clear_action:
                return startActivity(MainActivity.this, SettingsActivity.class);

            case R.id.map_action:
                return startActivity(MainActivity.this, MapsActivity.class);

            case R.id.send_message_action:
                return startActivity(MainActivity.this, MessageSendActivity.class);

        }
        return false;
    }

    /**
     * Extracts gui items to variables to be better accessible.
     */
    private void extractGuiItems() {
        mDataText = (TextView) findViewById(R.id.data_text);
        mStatusText = (TextView) findViewById(R.id.status_text);
        messagesListView = (ListView) findViewById(R.id.messagesListView);
        mWelcomeText = (TextView) findViewById(R.id.intro_text);
        String systemid = sharedPrefs.getString("id", null);
        if (systemid != null) {
            mWelcomeText.setText(R.string.welcome_message_registered);
        }
        if (!Network.isNetworkAvailable()) {
            mWelcomeText.append("\n" + getString(R.string.turn_on_internet_connection));
        }
    }

    /**
     * Sets or requests permissions for the application.
     */
    private void setPermissions() {
        boolean permissionsNotSets = true;
        while (permissionsNotSets) {
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                permissionsNotSets = true;
                ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION,
                        Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_EXTERNAL_STORAGE}, 0);
            } else {
                permissionsNotSets = false;
            }
        }
    }


    /**
     * Resets items to empty or zero values.
     */
    private void resetItems() {

        lat = 0;
        lon = 0;
        positionCount = 0;
        loggedPositionCount = 0;
        sendPositionCount = 0;
        messagesCount = 0;
        errorsCount = 0;

        mode = RequestMode.TRACKING;

        sessionId = null;
        waypoints = new ArrayList<>();
        searchid = sharedPrefs.getString("searchid", getString(R.string.pref_default_searchid));

        String[] messages = new String[]{getString(R.string.messages_list_title)};
        messages_list = new ArrayList<>(Arrays.asList(messages));

        Message mf = new Message(getString(R.string.messages_list_title), getString(R.string.messages_no_attachment), "");
        messages_list_full = new ArrayList<>();
        messages_list_full.add(mf);

        arrayAdapter = new ArrayAdapter<>
                (this, android.R.layout.simple_list_item_1, messages_list);

        // DataBind ListView with items from ArrayAdapter
        messagesListView.setAdapter(arrayAdapter);

        messagesListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                Intent appInfo = new Intent(MainActivity.this, MessageViewActivity.class);
                appInfo.putExtra("message", messages_list_full.get(position).getMessage());
                appInfo.putExtra("filename", messages_list_full.get(position).getFilename());
                appInfo.putExtra("from", messages_list_full.get(position).getFrom());
                startActivity(appInfo);
            }

        });
    }

    /**
     * It is triggered when the download of the data from Download is finished.
     *
     * @param result response from the server
     */
    public void processResponse(String result) {
        LogHelper.i("RESULT", result);
        switch (mode) {
            case SLEEPING:
                processSearchesResponse(result);
                break;
            case WAITING:
                processCallOnDutyResponse(result);
                break;
            case SELECTED:
                processSelectedResponse(result);
                break;
            case READY_FOR_TRACKING:
                processReadyForTrackingResponse(result);
                break;
            case TRACKING:
                processTrackingResponse(result);
                break;
            default:
                mDataText.setText(R.string.error_unexpected);
        }

    }

    /**
     * Reads results from server in mode tracking.
     * There can be several options.
     * ID, M, P and null.
     * ID - we have obtained session identifier from the server.
     * M - we have obtained message from server.
     * P - we have obtained information that position was saved.
     * ! - we have been released from duty
     * null - some error occured
     *
     * @param result result to process
     */
    private void processTrackingResponse(String result) {
        if (result != null) {
            if (result.startsWith("ID:")) {
                // We have obtained session id, so tour first location is saved as well
                sendPositionCount++;
                sendingLocation = false;
                sessionId = result.substring(3);
            } else if (result.startsWith("M")) {
                // We have obtained new message
                readingMessage = false;
                processMessage(result);
            } else if (result.startsWith("P")) {
                // We have obtained information about number of saved positions
                // We do not check it, it should be OK.
                sendingLocation = false;
                sendPositionCount = loggedPositionCount;
                setInfo();
            } else if (result.startsWith("!")) {
                goToSleep();
            }
        } else {
            sendingLocation = false;
            readingMessage = false;
            showErrorConnection();
        }
    }

    private void showErrorConnection() {
        //The response is null
        errorsCount++;
        DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
        Date date = new Date();
        mStatusText.append("\n" + getString(R.string.connection_error) + " v " + dateFormat.format(date) + ". Celkem chyb: " + errorsCount);
        LogHelper.e("ERROR", "Result is null");
        setInfo();
    }

    private void processReadyForTrackingResponse(String result) {
        if (result != null) {
            if (result.startsWith("<>")) {
                notificator.playRing();
                Toast toast = Toast.makeText(MainActivity.this, R.string.on_duty, Toast.LENGTH_LONG);
                toast.show();
                connect();
                menu.findItem(R.id.connect_disconnect_action).setTitle(R.string.disconnect);
            } else if (result.startsWith("!")) {
                goToSleep();
            }
        } else {
            //The response is null
            showErrorConnection();
        }
    }

    /**
     * Reads response in sleeping mode.
     * May contain have four states: *, #, empty, null
     * * - there is at least one active search on the server
     * # - the position of device was stored to the database, so we can switch to waiting mode
     * empty - nothing to do
     * null - some error occurred - nothing to do
     *
     * @param result result to process
     */
    private void processSearchesResponse(String result) {
        if (result != null) {
            if (!result.isEmpty() && RequestMode.SLEEPING == mode) {
                if (result.startsWith("*")) {
                    // there is a new search
                    String id = sharedPrefs.getString("id", null);
                    if (id != null && Network.isNetworkAvailable()) {
                        sendGetRequest(endPoint + "operation=searches&id=" + id + "&lat=" + getShortCoord(latFromListener) + "&lon=" + getShortCoord(lonFromListener));
                    }
                }
                if (result.startsWith("#")) {
                    // the coordinates where saved at the server
                    mode = RequestMode.WAITING;
                    mDataText.setText(R.string.mode_waiting);
                    // starts new timer with more often check
                    setCallOnDutyTimer();
                }
                if (result.startsWith("!")) {
                    // we are in sleeping mode, but server does not know
                    String id = sharedPrefs.getString("id", null);
                    if (id != null && Network.isNetworkAvailable()) {
                        sendGetRequest(endPoint + "operation=changestatus&status_to=sleeping&id=" + id);
                    }
                }
            }
            if (result.isEmpty()) {
                goToSleep();
            }
        }
    }

    /**
     * Reads response in waiting mode.
     * May contain three states: list, empty, null
     * list - list of active searches on the server
     * empty - no searches on call
     * null - some error occurred - nothing to do
     *
     * @param result result to process
     */
    private void processCallOnDutyResponse(String result) {
        if (result != null) {
            if (!result.isEmpty() && RequestMode.WAITING == mode) {
                // reads lists of searches
                String[] searches = result.split("\n");
                // TODO do it better
                if (callOnDuty == null) {
                    notificator.playRing();
                    callOnDuty = new Intent(MainActivity.this, CallOnDutyActivity.class);
                    callOnDuty.putExtra("searches", searches);
                    startActivity(callOnDuty);
                } else {
                    callOnDuty.setFlags(Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT);
                }
            }
        }
    }

    /**
     * Reads response in selected mode. User has selected the search.
     * May contain two states: some data, null
     * some data - server knows that you are ready
     * null - some error occurred - nothing to do TODO - do something, otherwise the call is not connected
     *
     * @param result result to process
     */
    private void processSelectedResponse(String result) {
        if (result != null) {
            if (!result.isEmpty() && RequestMode.SELECTED == mode) {
                // the searchid was saved at the server
                if (result.startsWith("@")) {
                    SharedPreferences.Editor editor = sharedPrefs.edit();
                    editor.putString("searchid", searchid);
                    editor.commit();
                    mode = RequestMode.READY_FOR_TRACKING;
                    mDataText.setText(R.string.mode_ready_for_tracking);
                } else if (result.startsWith("!")) {
                    mode = RequestMode.SLEEPING;
                    mDataText.setText(R.string.mode_sleeping);
                }
            }
        }
    }

    /**
     * Timer for sleeping and waiting mode.
     */
    private void setSearchTimer() {
        if (timerSearch != null) {
            timerSearch.cancel();
        }

        timerSearch = new Timer();
        mySearchTask = new SearchTask();
        timerSearch.schedule(mySearchTask, 30000, 1000 * 60 * 1);
    }

    /**
     * Timer for call on duty state.
     */
    private void setCallOnDutyTimer() {
        if (timerCallOnDuty != null) {
            timerCallOnDuty.cancel();
        }

        timerCallOnDuty = new Timer();
        myCallOnDutyTask = new CallOnDutyTask();
        timerCallOnDuty.schedule(myCallOnDutyTask, 0, 1000 * 5);
    }

    private void goToSleep() {
        String id = sharedPrefs.getString("id", null);
        if (id != null && Network.isNetworkAvailable()) {
            sendGetRequest(endPoint + "operation=changestatus&status_to=sleeping&id=" + id);
            disconnect();
            menu.findItem(R.id.connect_disconnect_action).setTitle(R.string.connect);
        }
    }

    /**
     * Checks new search.
     */
    private void checkSearches() {
        String id = sharedPrefs.getString("id", null);
        if (id != null
                && (RequestMode.SLEEPING == mode
                || RequestMode.READY_FOR_TRACKING == mode
                || RequestMode.TRACKING == mode) && Network.isNetworkAvailable()) {
            mWelcomeText.setText(R.string.welcome_message_registered);
            sendGetRequest(endPoint + "operation=searches&id=" + id);
        }
        if (!Network.isNetworkAvailable()) {
            Toast toast = Toast.makeText(MainActivity.this, R.string.turn_on_internet_connection, Toast.LENGTH_LONG);
            toast.show();
        }
    }

    /**
     * Check call on duty request.
     */
    public void checkCallOnDuty() {
        String id = sharedPrefs.getString("id", null);
        if (id != null && RequestMode.WAITING == mode && Network.isNetworkAvailable()) {
            sendGetRequest(endPoint + "operation=searches&id=" + id);
        }
        if (id != null && RequestMode.SELECTED == mode && Network.isNetworkAvailable()) {
            sendGetRequest(endPoint + "operation=searches&id=" + id + "&searchid=" + searchid + "&arrive=" + arrive);
        }
        if (id != null && RequestMode.READY_FOR_TRACKING == mode && Network.isNetworkAvailable()) {
            sendGetRequest(endPoint + "operation=searches&id=" + id);
        }
    }


    /**
     * Initialize the connection.
     */
    private void connect() {
        resetItems();

        // Sets the timer for logging position.
        if (timerPosition != null) {
            timerPosition.cancel();
        }
        timerPosition = new Timer();
        myPositionTask = new PositionTask();
        String sync_frequency_gps_string = sharedPrefs.getString("sync_frequency_gps", "5");
        int sync_frequency_gps = Integer.parseInt(sync_frequency_gps_string) * 1000;
        timerPosition.schedule(myPositionTask, 1000, sync_frequency_gps);


        // Sets the times for checking messages.
        if (timerMessage != null) {
            timerMessage.cancel();
        }

        timerMessage = new Timer();
        myMessageTask = new MessageTask();
        String sync_frequency_string = sharedPrefs.getString("sync_frequency", "30");
        int sync_frequency = Integer.parseInt(sync_frequency_string) * 1000;
        timerMessage.schedule(myMessageTask, 30000, sync_frequency);

        // set the state to connected
        connected = true;
    }

    /**
     * Stops timers.
     */
    private void disconnect() {
        connected = false;
        if (timerPosition != null) {
            timerPosition.cancel();
        }
        if (timerMessage != null) {
            timerMessage.cancel();
        }
        callOnDuty = null;
        mode = RequestMode.SLEEPING;
        mDataText.setText(R.string.mode_sleeping);
    }

    /**
     * Requests for session id.
     */
    private void getSessionId() {
        //Maybe put phone number instead of NN and random
        String systemid = sharedPrefs.getString("id", null);
        String user_name = sharedPrefs.getString("user_name", "NN " + Math.round(Math.random() * 10000));
        try {
            if (Network.isNetworkAvailable()) {
                sendGetRequest(endPoint + "operation=getid&searchid=" + searchid + "&user_name=" + URLEncoder.encode(user_name, "UTF-8") + "&systemid=" + systemid + "&lat=" + getShortCoord(lat) + "&lon=" + getShortCoord(lon));
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    /**
     * Reads location from system and compares it to the previous position.
     *
     * @return true if the position should be logged
     * @throws SecurityException
     */
    private boolean trackLocation() throws SecurityException {
        double newlat = 0;
        double newlon = 0;
        boolean logit = false;
        Location locationGPS = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
        if (locationGPS == null) {
            // we do not have location yet, so wait for location to the next interval
            return false;
        }
        newlat = locationGPS.getLatitude();
        newlon = locationGPS.getLongitude();
        positionCount++;
        //TODO do it better
        if (Math.hypot(lat - newlat, lon - newlon) >= 0.0001) { //0.0001
            lat = newlat;
            lon = newlon;
            logit = true;
            Waypoint wp = new Waypoint(newlat, newlon, new Date());
            waypoints.add(wp);
            loggedPositionCount++;
            lastLoggedPositionId++;
        }
        setInfo();
        return logit;
    }

    /**
     * Sends current location or cached 100 locations to the server.
     */
    private void sendTrack() {
        if ((sendPositionCount == (loggedPositionCount - 1)) && Network.isNetworkAvailable()) {
            sendingLocation = true;
            sendGetRequest(endPoint + "operation=sendlocation&searchid=" + searchid + "&id=" + sessionId + "&lat=" + getShortCoord(lat) + "&lon=" + getShortCoord(lon));
        }
        if ((sendPositionCount < (loggedPositionCount - 1)) && Network.isNetworkAvailable()) {
            //Some time out of network. Must send more coords from memory.
            //TODO change to POST to be able send more than 100 coords
            String notSendedCoords = "";
            int startPosition = sendPositionCount + 1;
            //When is more than 50 not sent positions in memory then points are generalized to last 100 is sent to server
            if ((MainActivity.waypoints.size() - startPosition) > 50) {
                int step = (int) Math.ceil((MainActivity.waypoints.size() - startPosition) / 50);
                for (int i = startPosition; i < MainActivity.waypoints.size(); i+=step) {
                    Waypoint wp = MainActivity.waypoints.get(i);
                    notSendedCoords += getShortCoord(wp.getLon()) + ";" + getShortCoord(wp.getLat()) + ",";
                }
            } else {
                for (int i = startPosition; i < MainActivity.waypoints.size(); i++) {
                    Waypoint wp = MainActivity.waypoints.get(i);
                    notSendedCoords += getShortCoord(wp.getLon()) + ";" + getShortCoord(wp.getLat()) + ",";
                }
            }
            if (notSendedCoords.length() > 10) {
                notSendedCoords = notSendedCoords.substring(0, notSendedCoords.length() - 2);
                sendingLocation = true;
                sendGetRequest(endPoint + "operation=sendlocations&searchid=" + searchid + "&id=" + sessionId + "&coords=" + notSendedCoords);
            }
        }
    }

    /**
     * Shows the information that the posision is the same.
     */
    private void showInfoSamePosition() {
        setInfo();
        String content = String.valueOf(mDataText.getText());
        DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
        Date date = new Date();
        content += " " + getString(R.string.same_position_label) + ": " + dateFormat.format(date);
        mDataText.setText(content);
    }

    /**
     * Sends coordinates to the server.
     *
     * @throws SecurityException
     */
    private void sendCoordinates() throws SecurityException {
        boolean logit = trackLocation();
        if (MainActivity.StatusMessages != null) {
            mStatusText.append("\n" + MainActivity.StatusMessages);
            MainActivity.StatusMessages = null;
        }
        // First we have to obtain the sessionId.
        if (sessionId == null) {
            getSessionId();
        } else {
            // if the position has not changed we do not send it to the server
            if (logit) {
                if (!sendingLocation) {
                    sendTrack();
                }
            } else {
                showInfoSamePosition();
            }
        }
    }

    /**
     * Flat the coordinate to has just 6 decimal points.
     *
     * @param coord coordinate to flat
     * @return flatted coordinate
     */
    private String getShortCoord(double coord) {
        String coordLong = Double.toString(coord);
        String parts[] = coordLong.split("\\.");
        if (parts[1].length() > 6) {
            return parts[0] + "." + parts[1].substring(0, 6);
        } else {
            return parts[0] + "." + parts[1];
        }
    }

    /**
     * Starts downloading a message.
     */
    private void downloadMessage() {
        // we do not have session id yet, so ask for it
        if (sessionId == null && Network.isNetworkAvailable()) {
            sendGetRequest(endPoint + "operation=getid&searchid=" + searchid);
        } else {
            boolean messages_switch = sharedPrefs.getBoolean("messages_switch", true);
            if (messages_switch && !readingMessage && Network.isNetworkAvailable()) {
                readingMessage = true;
                sendGetRequest(endPoint + "operation=getmessages&searchid=" + searchid + "&id=" + sessionId);
            }
        }
    }

    /**
     * Process the new message. If there is an attachment it is downloaded.
     *
     * @param result reponse from the server
     */
    private void processMessage(String result) {
        //New mesage is on the way
        messagesCount++;
        String items[] = result.split(";");
        Message mf = new Message(getString(R.string.message_when) + ": " + items[4] + "\n" + getString(R.string.message) + ": " + items[2], items[3], items[6]);
        messages_list_full.add(0, mf);
        String short_message = items[2].length() > 20 ? items[2].substring(0, 20) + "..." : items[2];
        if (items[3].length() > 1) {
            messages_list.add(0, items[6] + ": " + items[4].substring(0, items[4].length() - 3).split(" ")[1] + ": " + short_message + " (@)");
            //Shared file
            String shared = items[5].replace("\n", "");
            if (shared.equalsIgnoreCase("1")) {
                if (Network.isNetworkAvailable()) {
                    downloadFromUrl(endPoint + "operation=getfile&searchid=" + searchid + "&id=shared&filename=" + items[3], items[3]);
                }
            } else {
                //Individual file
                if (Network.isNetworkAvailable()) {
                    downloadFromUrl(endPoint + "operation=getfile&searchid=" + searchid + "&id=" + sessionId + "&filename=" + items[3], items[3]);
                }
            }
        } else {
            messages_list.add(0, items[6] + ": " + items[4].substring(0, items[4].length() - 3).split(" ")[1] + ": " + short_message);
        }
        new AdapterHelper().update((ArrayAdapter) arrayAdapter, new ArrayList<Object>(messages_list));
        arrayAdapter.notifyDataSetChanged();
        notificator.playRing();
    }

    /**
     * Shows the information in the status bar.
     */
    private void setInfo() {
        String content = getString(R.string.sessionid_label) + ": "
                + sessionId + " "
                + getString(R.string.searchid_label)
                + ": " + searchid + "\n";
        content += getString(R.string.positions_label)
                + ": " + positionCount
                + "/" + loggedPositionCount
                + "/" + sendPositionCount + "\n";
        content += getString(R.string.longitude_label) + ": "
                + getShortCoord(lon)
                + " " + getString(R.string.latitude_label) + ": "
                + getShortCoord(lat)
                + "\n";
        mDataText.setText(content);
    }

    /**
     * Starts the new activity.
     *
     * @param packageContext context of the package
     * @param appToStart application to start
     *
     * @return true
     */
    private boolean startActivity(Context packageContext, Class<?> appToStart) {
        Intent appToStartIntent = new Intent(packageContext, appToStart);
        startActivity(appToStartIntent);
        return true;
    }

    /**
     * Downloads file from url.
     *
     * @param url url path to file
     * @param file filename where to save it
     */
    private void downloadFromUrl(String url, String file) {
        DownloadFileFromUrl downloadFileFromUrl = new DownloadFileFromUrl();
        downloadFileFromUrl.execute(url, file);
    }

    /**
     * Sends request via GET to url.
     *
     * @param url where to send
     */
    private void sendGetRequest(String url) {
        GetRequest getRequest = new GetRequest();
        getRequest.setActivity(this);
        getRequest.execute(url);
        //mStatusText.append("\n" + url.substring(url.indexOf("?")));
        LogHelper.i("REQUEST", url.substring(url.indexOf("?")));
    }

    /**
     * Timer task for synchronization of positions.
     */
    class PositionTask extends TimerTask {

        @Override
        public void run() {
            runOnUiThread(new Runnable() {

                @Override
                public void run() {
                   sendCoordinates();
                }
            });
        }

    }

    /**
     * Timer task for synchronization of messages.
     */
    class MessageTask extends TimerTask {

        @Override
        public void run() {
            runOnUiThread(new Runnable() {

                @Override
                public void run() {
                    downloadMessage();
                }
            });
        }

    }

    /**
     * Timer task for synchronization of messages.
     */
    class SearchTask extends TimerTask {

        @Override
        public void run() {
            runOnUiThread(new Runnable() {

                @Override
                public void run() {
                    checkSearches();
                }
            });
        }

    }

    /**
     * Timer task for synchronization of messages.
     */
    class CallOnDutyTask extends TimerTask {

        @Override
        public void run() {
            runOnUiThread(new Runnable() {

                @Override
                public void run() {
                    checkCallOnDuty();
                }
            });
        }

    }
}

